<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://welaunch.io/plugins/wordpress-multilingual-multisite/
 * @since      1.0.0
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 * @author     Daniel Barenkamp <support@welaunch.io>
 */
class WordPress_Multilingual_Multisite_Importer extends WordPress_Multilingual_Multisite {
    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $plugin_name    The ID of this plugin.
     */
    protected $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $version    The current version of this plugin.
     */
    protected $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of this plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct( $plugin_name, $version, $current_blog_id) 
    {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
        $this->current_blog_id = $current_blog_id;
    }

    public function init()
    {
        global $wordpress_multilingual_multisite;

        $this->options = $wordpress_multilingual_multisite;

        if (!$this->get_option('enable')) {
            return FALSE;
        }

        if (!$this->get_option('postImportExportEnable')) {
            return FALSE;
        }

        add_action('admin_menu', array($this, 'create_importer_menu'));
    }

    public function create_importer_menu() {

        add_submenu_page(
            'wordpress_multilingual_multisite_options',
            esc_html__('Import Posts', 'wordpress-multilingual-multisite'),
            esc_html__('Import Posts', 'wordpress-multilingual-multisite'),
            'manage_options',
            'wordpress-multilingual-multisite-importer',
            array($this, 'settings_page')
        );
    }

    public function settings_page() {
        if( !current_user_can('administrator') )  {
            return false;
        }

        $this->check_file_uploaded();
    ?>
        <div class="wrap">
            <h1><?php echo esc_html__('Import Posts', 'wordpress-multilingual-multisite') ?></h1>

            <form method="post" enctype="multipart/form-data">

                <table class="form-table">       
                    <tr valign="top">
                        <th scope="row"><?php esc_html_e('File to Import', 'wordpress-multilingual-multisite') ?></th>
                        <td><input type='file' id='import_file' name='import_file'></input></td>
                    </tr>
                    <tr valign="top">
                        <th scope="row"><?php esc_html_e('Try Update', 'wordpress-multilingual-multisite') ?></th>
                        <td><input type="checkbox" name="update_data" checked="checked"> <?php esc_html_e('Try Updating Data check will be done by ID. If an ID exists in your file the ID will be updated.If ID not exists a post will be created.', 'wordpress-multilingual-multisite') ?></td>
                    </tr>
                </table>

                <p class="submit">
                    <input type="submit" name="wordpress-multilingual-multisite-import" id="submit" class="button button-primary" value="Import">
                </p>
            </form>
        </div>
    <?php 
    }

    public function check_file_uploaded()
    {
        if(!isset($_POST) || empty($_POST)) {
            return FALSE;
        }

        if(!isset($_POST['wordpress-multilingual-multisite-import']) || empty($_POST['wordpress-multilingual-multisite-import'])) {
            return FALSE;
        }

        if(!isset($_FILES['import_file']['name']) || empty($_FILES['import_file']['name'])){
            $this->notice = "No file selected.";
            $this->notice();
            return FALSE;
        }

        if(isset($_POST['update_data'])) {
            $this->try_update = true;
        }

        // Check allowed mime types
        $xls_mimetypes = array();
        if($this->get_option('postImportExportFormat') == "json") {
            $xls_mimetypes[] = 'application/json';
        } elseif ($this->get_option('postImportExportFormat') == "xml") {
            $xls_mimetypes[] = 'text/xml';
        }

        if(!in_array($_FILES['import_file']['type'], $xls_mimetypes)){
            $this->notice =  "Only " . $this->get_option('postImportExportFormat') . " files allowed (given: ".$_FILES["import_file"]["type"].")";
            $this->notice();
            return FALSE;
        }

        // Use the wordpress function to upload
        // import_file corresponds to the position in the $_FILES array
        // 0 means the content is not associated with any other posts
        $attachment_id = media_handle_upload('import_file', 0);
        $link = get_attached_file($attachment_id);

        // Error checking using WP functions
        if(is_wp_error($attachment_id)){
            $this->notice = "Error uploading file: " . $attachment_id->get_error_message();
            $this->notice();
            return FALSE;
        }


        if($this->get_option('postImportExportFormat') == "json") {
            $data = json_decode( file_get_contents($link) );
        } elseif ($this->get_option('postImportExportFormat') == "xml") {
            $data = simplexml_load_string( file_get_contents($link) );
        }

        $this->handle_upload($data);
    }

    public function handle_upload($data)
    {
        // Check target blog exists, else use current one
        if(empty($data->target_blog)) {
            $target_blog = $this->current_blog_id;
        } else {
            $target_blog = $data->target_blog;
        }

        $source_blog = $data->source_blog;

        if(empty($data->posts)) {
            die( esc_html__('No posts found!', 'wordpress-multilingual-multisite') );
        }

        try {
            switch_to_blog($target_blog);

            $posts = $data->posts;

            $i = 0;
            foreach ($posts as $post) {

                if(isset($post->item0)) {
                    $post = $post->item0;
                }
                
                unset($post->post_author);

                $check_exists = 0;
                if($this->try_update) {

                    if(!empty($post->ID)) {

                        $check_exists = get_post($post->ID);
                        if(!empty($check_exists)) {
                            $post->ID = $check_exists->ID;
                            $post_id = wp_update_post($post, true);
                        } else {
                            unset($post->ID);
                            $check_exists = 0;
                            $post_id = wp_insert_post($post, true);
                        }
                    } else {
                        $this->notice .= esc_html__('Post ID missing.', 'wordpress-multilingual-multisite');
                    }
                } else {
                    unset($post->ID);
                    $post_id = wp_insert_post($post, true);
                }


                if(is_wp_error($post_id)) {
                    $this->notice .= 'Post: ' . $post->post_title . ' error: <br/>' . print_r($post_id, true) . '<br/><br/>'; //$post_id->get_error_m‌​essage();
                    continue;
                } else {
                    if($check_exists !== 0) {
                        $this->notice .= 'Post ' . $post_id . ': ' . $post->post_title . ' successfully updated<br/>';
                    } else {
                        $this->notice .= 'Post ' . $post_id . ': ' . $post->post_title . ' successfully imported<br/>';
                    }

                    if(!empty($post->metas)) {
                        foreach ($post->metas as $post_meta_key => $post_meta_value) {
                            update_post_meta($post_id, $post_meta_key, $post_meta_value[0]);
                        }
                    }
                }

            }
        } catch (Exception $e) {
            $this->notice = 'Your file seems to be corrupt.<br/>' . $e->getMessage();
        }

        $this->notice();
    }

    public function notice()
    {
        ?>
        <div class="notice notice-success is-dismissible">
            <p><?php echo wp_kses_post($this->notice) ?></p>
        </div>
        <?php
    }

    public function allow_json_xml_mime_types( $mimes ) 
    {
        // New allowed mime types.
        $mimes['json'] = 'application/json';
        $mimes['xml'] = 'text/xml';

        return $mimes;
    }

}