<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://welaunch.io/plugins/wordpress-multilingual-multisite/
 * @since      1.0.0
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/includes
 * @author     Daniel Barenkamp <support@welaunch.io>
 */
class WordPress_Multilingual_Multisite {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      WordPress_Multilingual_Multisite_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */

	public function __construct() 
	{

		$this->plugin_name = 'wordpress-multilingual-multisite';
		$this->version = '1.0.0';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - WordPress_Multilingual_Multisite_Loader. Orchestrates the hooks of the plugin.
	 * - WordPress_Multilingual_Multisite_i18n. Defines internationalization functionality.
	 * - WordPress_Multilingual_Multisite_Admin. Defines all hooks for the admin area.
	 * - WordPress_Multilingual_Multisite_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() 
	{
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'vendor/autoload.php';

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wordpress-multilingual-multisite-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wordpress-multilingual-multisite-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-multilingual-multisite-admin.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-multilingual-multisite-blog-options.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-multilingual-multisite-blog-copy.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-multilingual-multisite-export.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-multilingual-multisite-importer.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-multilingual-multisite-machine-translation.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-multilingual-multisite-media.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-multilingual-multisite-network-query.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-multilingual-multisite-post-copier.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-multilingual-multisite-posts-translations.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-multilingual-multisite-post-overview.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-multilingual-multisite-public.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-multilingual-multisite-sync.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-multilingual-multisite-terms-translations.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-multilingual-multisite-terms-copier.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-multilingual-multisite-translation-tables.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-multilingual-multisite-translator.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-multilingual-multisite-users.php';		

		$this->loader = new WordPress_Multilingual_Multisite_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the WordPress_Multilingual_Multisite_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() 
	{

		$plugin_i18n = new WordPress_Multilingual_Multisite_i18n();
		$this->loader->add_action('plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_hooks() 
	{
		$current_blog_id = get_current_blog_id();

		// add_action('init', array($this, 'load_data'), 0);
		$this->load_data();

        // Translation Tables
        $translation_tables = new WordPress_Multilingual_Multisite_Translation_Tables( $this->get_plugin_name(), $this->get_version(), $current_blog_id );
        $this->loader->add_action('wp_insert_site', $translation_tables, 'add_blog_to_translation_table', 10, 1 );
        $this->loader->add_action('delete_blog', $translation_tables, 'remove_blog_from_translation_table', 10, 2 );

		$plugin_admin = new WordPress_Multilingual_Multisite_Admin( $this->get_plugin_name(), $this->get_version(), $current_blog_id, $translation_tables );
		$this->loader->add_action('plugins_loaded', $plugin_admin, 'load_redux' );
	  	$this->loader->add_action('init', $plugin_admin, 'init', 1);
	  	$this->loader->add_action('init', $plugin_admin, 'set_transients', 2 );
	  	$this->loader->add_action('update_site_option', $plugin_admin, 'delete_transient' );
	  	$this->loader->add_action('update_option', $plugin_admin, 'delete_transient' );
	  	$this->loader->add_action('admin_bar_menu', $plugin_admin, 'add_admin_bar_menu', 100 );

        $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_styles', 999 );
        $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts', 999 );

        // Network Query
	  	$network_query = new WordPress_Multilingual_Multisite_Network_Query( $this->get_plugin_name(), $this->get_version() );
	  	$this->loader->add_action('init', $network_query, 'init', 2);

	  	// Blog Copy
	  	$blog_copy = new WordPress_Multilingual_Multisite_Blog_Copy( $this->get_plugin_name(), $this->get_version(), $network_query, $translation_tables, $current_blog_id );
	  	$this->loader->add_action( 'init', $blog_copy, 'init', 2);
		$this->loader->add_action( 'network_admin_menu', $blog_copy, 'add_menu' );
		$this->loader->add_filter( 'manage_sites_action_links', $blog_copy, 'add_site_action', 10, 2 );

	  	// Post Translator Linking
	  	$posts_translations = new WordPress_Multilingual_Multisite_Posts_Translations( $this->get_plugin_name(), $this->get_version(), $network_query, $translation_tables, $current_blog_id);
	  	$this->loader->add_action('init', $posts_translations, 'init', 5 );
        $this->loader->add_action('add_meta_boxes', $posts_translations, 'add_custom_metaboxes', 10, 2);
        $this->loader->add_action('post_updated', $posts_translations, 'save_custom_metaboxes', 3, 3);
        
		// $this->loader->add_filter('manage_posts_columns', $posts_translations, 'modify_columns_head');
		// $this->loader->add_action('manage_posts_custom_column', $posts_translations, 'modify_columns_content', 10, 2);
		// $this->loader->add_filter('manage_page_posts_columns', $posts_translations, 'modify_columns_head', 10);
		// $this->loader->add_action('manage_page_posts_custom_column', $posts_translations, 'modify_columns_content', 10, 2);

		$this->loader->add_action('wp_ajax_wordpress_multilingual_multisite_get_network_posts', $posts_translations, 'ajax_get_network_posts');
		

		if(isset($_GET['init-translation-tables'])) {
			$this->loader->add_action('init', $posts_translations, 'init_translation_tables', 10, 2);
		}

	  	// Terms Translations
	  	$terms_translations = new WordPress_Multilingual_Multisite_Terms_Translations( $this->get_plugin_name(), $this->get_version(), $network_query, $translation_tables, $current_blog_id);
		$this->loader->add_action('init', $terms_translations, 'init', 9999);

		// Importer
	  	$importer = new WordPress_Multilingual_Multisite_Importer( $this->get_plugin_name(), $this->get_version(), $current_blog_id);
	  	$this->loader->add_action('init', $importer, 'init', 10 );
	  	$this->loader->add_filter('upload_mimes', $importer, 'allow_json_xml_mime_types', 10 );

	  	// Media Sync
	  	$media = new WordPress_Multilingual_Multisite_Media( $this->get_plugin_name(), $this->get_version(), $network_query, $translation_tables, $current_blog_id);
	  	$this->loader->add_action('init', $media, 'init', 10 );

		// Machine Translation
	  	$machine_translation = new WordPress_Multilingual_Multisite_Machine_Translation( $this->get_plugin_name(), $this->get_version(), $current_blog_id);
	  	$this->loader->add_action('init', $machine_translation, 'init', 80 );
        $this->loader->add_action('add_meta_boxes', $machine_translation, 'add_custom_metaboxes', 10, 2);
        $this->loader->add_action('save_post', $machine_translation, 'translate_single_post', 1, 5);
        $this->loader->add_action('init', $machine_translation, 'translate_multiple_posts', 90);
        $this->loader->add_action('init', $machine_translation, 'translate_multiple_terms', 90);
        $this->loader->add_action('init', $machine_translation, 'translate_multiple_comments', 90);

		// Translator Role
	  	$translator = new WordPress_Multilingual_Multisite_Translator( $this->get_plugin_name(), $this->get_version(), $current_blog_id);
	  	$this->loader->add_action('init', $translator, 'init', 10 );
        $this->loader->add_action('save_post', $translator, 'override_post_status', 1, 5);
		$this->loader->add_action('add_meta_boxes', $translator, 'add_translator_meta_box', 10, 1);
		$this->loader->add_action('save_post', $translator, 'save_translator', 50, 3);
		$this->loader->add_action('pre_get_posts', $translator, 'filter_posts_by_translator', 10, 1 );
		$this->loader->add_filter('admin_body_class', $translator, 'add_translator_class', 10 );
		$this->loader->add_action('admin_init', $translator, 'remove_menu_pages', 10 );		

        // Network Post Overview Pages
	  	$post_overview = new WordPress_Multilingual_Multisite_Post_Overview( $this->get_plugin_name(), $this->get_version(), $network_query, $translation_tables);
        $this->loader->add_action('network_admin_menu', $post_overview, 'add_network_pages', 10);

        // Terms Copier
    	$terms_copier = new WordPress_Multilingual_Multisite_Terms_Copier( $this->get_plugin_name(), $this->get_version(), $translation_tables, $current_blog_id );
    	$this->loader->add_action('init', $terms_copier, 'init', 5 );
    	$this->loader->add_action('init', $terms_copier, 'copy', 10 );

        // Post Copier
    	$post_copier = new WordPress_Multilingual_Multisite_Post_Copier( $this->get_plugin_name(), $this->get_version(), $translation_tables, $terms_copier, $current_blog_id );
    	$this->loader->add_action('init', $post_copier, 'init', 5 );
    	$this->loader->add_action('init', $post_copier, 'copy_single', 10 );
    	$this->loader->add_action('init', $post_copier, 'copy_multiple', 10 );

    	// Public (alternate tags, widget etc.)
    	$plugin_public = new WordPress_Multilingual_Multisite_Public( $this->get_plugin_name(), $this->get_version(), $translation_tables, $current_blog_id );
		$this->loader->add_action('wp', $plugin_public, 'init', 10 );
    	$this->loader->add_action('wp_head', $plugin_public, 'add_alternate_tags', 10 );
    	$this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_styles', 10 );
    	$this->loader->add_filter('wp_nav_menu_items', $plugin_public, 'add_country_selector_to_menu', 10, 2);
    	$this->loader->add_filter('language_attributes', $plugin_public, 'change_language_attributes', 10, 2);
    	

    	// Blog Options
    	$blog_options = new WordPress_Multilingual_Multisite_Blog_Options( $this->get_plugin_name(), $this->get_version(), $current_blog_id );
    	$this->loader->add_action('init', $blog_options, 'init', 5);
    	// $this->loader->add_action('wp_insert_site', $blog_options, 'add_options_to_new_blog', 20, 1 );
    	$this->loader->add_action('admin_init', $blog_options, 'register_fields', 10 );

    	// Sync
    	$sync = new WordPress_Multilingual_Multisite_Sync( $this->get_plugin_name(), $this->get_version(), $translation_tables, $current_blog_id, $post_copier, $terms_copier );
    	$this->loader->add_action('init', $sync, 'init', 20, 6 );
    	$this->loader->add_action('updated_post_meta', $sync, 'sync_post_meta', 10, 4);
    	$this->loader->add_action('added_post_meta', $sync, 'sync_post_meta', 10, 4);
    	$this->loader->add_action('deleted_post_meta', $sync, 'delete_post_meta', 10, 4);
    	$this->loader->add_action('post_updated', $sync, 'sync_post_data', 10, 3);
    	$this->loader->add_action('post_updated', $sync, 'sync_post_terms', 15, 3);
    	$this->loader->add_action('transition_post_status', $sync, 'create_post', 10, 3);
    	$this->loader->add_action('before_delete_post', $sync, 'delete_post', 10, 1);
    	$this->loader->add_action('wp_trash_post', $sync, 'trash_post', 10, 1);

    	// Post Import Export
    	$exporter = new WordPress_Multilingual_Multisite_Export( $this->get_plugin_name(), $this->get_version(), $translation_tables, $current_blog_id );
    	$this->loader->add_action('init', $exporter, 'init', 10 );
    	$this->loader->add_action('init', $exporter, 'maybe_export', 10 );
    	$this->loader->add_action('add_meta_boxes', $exporter, 'add_metabox', 10, 2 );

    	// Users
    	$users = new WordPress_Multilingual_Multisite_Users( $this->get_plugin_name(), $this->get_version(), $current_blog_id );
    	$this->loader->add_action('init', $users, 'init', 10 );
    	$this->loader->add_action('user_register', $users, 'user_registered', 10 );
    	$this->loader->add_action('profile_update', $users, 'profile_updated', 10, 2 );
    	// $this->loader->add_filter('woocommerce_order_query', $users, 'test', 10, 2 );
    	// need to hook then into wc_get_order to check if exists in current site
    	// more problem: in multisite an order id can be duplicate
	}

	public function load_data()
	{
		// Data
		add_action('plugins_loaded', function() { 
			
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'data/continents.php';
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'data/countries.php';
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'data/countriesByContinents.php';
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'data/languages.php';
		}, 21);
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() 
	{
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() 
	{
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    WordPress_Multilingual_Multisite_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() 
	{
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() 
	{
		return $this->version;
	}

	
	/**
	 * Gets options
	 *
	 * @since    1.0.0
	 */
    protected function get_option($option)
    {
		if(!isset($this->options)) {
    		return false;
    	}

		if(!is_array($this->options)) {
    		return false;
    	}
    	
    	if(!array_key_exists($option, $this->options))
    	{
    		return false;
    	}
    	return $this->options[$option];
    }

    protected function get_current_user_role()
    {
		$user = wp_get_current_user();
	    $role = ( array ) $user->roles;

	    if(!isset($role[0])) {
	    	return false;
	    }

	    return $role[0];
    }
}